"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const proxyquire = require("proxyquire");
const assert = require("power-assert");
const sinon = require("sinon");
const connectStub = sinon.spy();
const MongoStub = {
    MongoClient: {
        connect: () => { }
    }
};
const MongoConnector = proxyquire('../mongo_connector', { 'mongodb': MongoStub }).default;
describe('connector', () => {
    it('will initialise url', () => {
        const url = 'mongodb://url';
        const spy = sinon.spy();
        MongoStub.MongoClient.connect = () => spy();
        const connector = new MongoConnector(url);
        connector.connect();
        assert.equal(connector.mongoUrl, url);
        assert(spy.calledOnce);
    });
    it('will not initialise db on error and logs error', sinon.test(function () {
        const url = 'mongodb://url';
        const successSpy = sinon.spy();
        const dirStub = this.stub(console, 'log');
        MongoStub.MongoClient.connect = (url, func) => func('error');
        const connector = new MongoConnector(url);
        connector.connect();
        assert.equal(connector.db, undefined);
        sinon.assert.calledOnce(dirStub);
        sinon.assert.calledWith(dirStub, 'Connection Error: error');
    }));
    it('will initialise and calls back', () => {
        const url = 'mongodb://url';
        const db = {};
        const startedSpy = sinon.spy();
        MongoStub.MongoClient.connect = (url, func) => func(null, db);
        const connector = new MongoConnector(url);
        connector.connect(startedSpy);
        assert.equal(connector.db, db);
        assert(startedSpy.calledOnce);
    });
    it('will create a new collection', () => __awaiter(this, void 0, void 0, function* () {
        const url = '';
        const db = {
            collection: sinon.spy()
        };
        MongoStub.MongoClient.connect = (url, func) => func(null, db);
        const connector = new MongoConnector(url);
        connector.connect();
        console.log(connector.db);
        assert.equal(connector.db, db);
        const collectionName = 'Col';
        const col = connector.collection(collectionName);
        assert(db.collection.calledWith(collectionName));
    }));
    it('can dispose database', () => __awaiter(this, void 0, void 0, function* () {
        const connector = new MongoConnector();
        connector.db = {
            dropDatabase: sinon.stub(),
            close: sinon.stub()
        };
        yield connector.dispose();
        sinon.assert.calledOnce(connector.db.dropDatabase);
        sinon.assert.calledOnce(connector.db.close);
    }));
});
